﻿using System.Threading;
using System.Windows;
using System.ComponentModel;

namespace BackgroundWorkerInWPF
{
    /// <summary>
    /// Interaction logic for WorkerWindow.xaml
    /// </summary>
    public partial class WorkerWindow : Window
    {
        private BackgroundWorker backgroundWorker;

        public WorkerWindow()
        {
            InitializeComponent();
            backgroundWorker = (BackgroundWorker)FindResource("backgroundWorker");
        }

        #region Long-Running Process

        private int DoSlowProcess(int iterations,
            BackgroundWorker worker, DoWorkEventArgs e)
        {
            // throw exception for testing
            // throw new System.Exception("Something bad happened");

            int result = 0;

            for (int i = 1; i <= iterations; i++)
            {
                if (worker != null)
                {
                    if (worker.CancellationPending)
                    {
                        e.Cancel = true;
                        break;
                    }
                    if (worker.WorkerReportsProgress)
                    {
                        int percentComplete = (int)((float)i / (float)iterations * 100);
                        string updateMessage =
                            string.Format("Iteration {0} of {1}", i, iterations);
                        worker.ReportProgress(percentComplete, updateMessage);
                    }
                }

                Thread.Sleep(100);
                result = i;
            }

            return result;
        }

        #endregion

        #region UI Events

        private void startButton_Click(object sender, RoutedEventArgs e)
        {
            int iterations = 0;
            if (int.TryParse(inputBox.Text, out iterations))
            {
                if (!backgroundWorker.IsBusy)
                    backgroundWorker.RunWorkerAsync(iterations);
                startButton.IsEnabled = false;
                cancelButton.IsEnabled = true;
                outputBox.Text = "";
            }
        }

        private void cancelButton_Click(object sender, RoutedEventArgs e)
        {
            backgroundWorker.CancelAsync();
        }

        #endregion

        #region BackgroundWorker Events

        private void BackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            var bgw = sender as BackgroundWorker;
            e.Result = DoSlowProcess((int)e.Argument, bgw, e);
        }

        private void BackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                outputBox.Text = e.Error.Message;
                MessageBox.Show(e.Error.StackTrace);
            }
            else if (e.Cancelled)
            {
                outputBox.Text = "Cancelled";
            }
            else
            {
                outputBox.Text = e.Result.ToString();
                mainProgressBar.Value = 0;
            }
            startButton.IsEnabled = true;
            cancelButton.IsEnabled = false;
        }

        private void BackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            mainProgressBar.Value = e.ProgressPercentage;
            outputBox.Text = (string)e.UserState;
        }

        #endregion
    }
}
